# Program to generate a scatter plot and find the best fit polynommial equation.
# for space shuttle ascent data files. The data is provided as a CSV file.
# I used space shuttle ascent data available at: 
# https://www.nasa.gov/pdf/522602main_AP_ED_ShuttleAscent_Nspire.pdf
#
# My goal here is provide a simple python script to generate all plots
# and regression models rquested in the activity described in the
# Math and Science @ Work material (https://www.nasa.gov/audience/foreducators/mathandscience/exploration/Prob_ShuttleAscent_detail.html)
#
# Nasa sites
# https://www.nasa.gov/audience/foreducators/mathandscience/home/index.html
# https://www.nasa.gov/audience/foreducators/mathandscience/exploration/Prob_ShuttleAscent_detail.html
# https://www.nasa.gov/audience/foreducators/mathandscience/exploration/index.html
# 
#
# Dr. Walter F. de Azevedo Jr
# azevedolab.net
# August, 24 2018.
#

# Create ModelAscent() Class

class ModelAscent(object):
    """Class to generate a model for space shuttle ascent data files"""
    
    # Constructor method
    def __init__(self,file_in,pol_deg,exp_var,resp_var):
        self.file_in = file_in      # Space shuttle ascent CSV file name
        self.pol_deg = pol_deg      # Degree of the polynomial equation
        self.exp_var = exp_var      # Column number with explanatory variable data
        self.resp_var = resp_var    # Column number with response variable data
    
    # Method to read space shuttle ascent CSV file name
    def read_data(self):
        """Method to read space shuttle ascent CSV file name"""
        
        # Import libraries
        import csv
        import numpy as np
        
        # Read headers
        fo = open(self.file_in,"r")
        csv1 = csv.reader(fo)

        # Looping through te first line to get headers
        for line in csv1:
            self.x_label = line[int(self.exp_var)-1]
            self.y_label = line[int(self.resp_var)-1]
            break

        # Close CSV file
        fo.close()
        
        # Some editing
        aux_title0 = str(self.y_label)
        aux_title1 = aux_title0.replace(" ","")
        aux_title2 = aux_title1.replace("(m)","")
        aux_title3 = aux_title2.replace("(s)","")
        aux_title4 = aux_title3.replace("(m/s)","")
        self.aux_title = aux_title4.replace("(m/s2)","")

        # Read CSV file to get numerical data
        my_csv = np.genfromtxt (self.file_in, delimiter=",", skip_header = 1)

        # Get each column from CSV file
        self.x = my_csv[:,self.exp_var-1]
        self.y = my_csv[:,self.resp_var-1]
    
    # Method to generate a regression model
    def regression_model(self):
        """Method to generate a regression model for space shuttle ascent"""
        
        # Import library
        import numpy as np
        
        # Least-squares polynomial fitting 
        self.z = np.polyfit(self.x,self.y,self.pol_deg)
        self.p = np.poly1d(self.z)
        pol_in_t_aux = str(self.p)
        pol_in_t = pol_in_t_aux.replace("x","t") # To show polynomial equation with t variable
        print("Best fit polynomial equation: ",pol_in_t)
    
    # Method to plot scatter graph
    def plot_data(self):
        """Method to generate plot"""
        
        # Import libraries
        import numpy as np
        import matplotlib.pyplot as plt
        
        # Set up x-axis limits
        plt.xlim(0,self.x[len(self.x)-1])

        # Generate scatter plot
        plt.scatter(self.x,self.y) 

        # Set up axis labels
        plt.xlabel(self.x_label) 
        plt.ylabel(self.y_label) 

        # Generate plot
        #plt.plot(self.x, p(self.x), '-')

        # Show plot
        plt.show()

        # Save plot on png file
        plt.savefig(self.file_in[:len(self.file_in)-4]+"_"+self.aux_title+".png")
    
    # Method to plot scatter graph with regression model
    def plot_model(self):
        """Method to generate plot of data and a regression model"""
        
        # Import libraries
        import numpy as np
        import matplotlib.pyplot as plt
        
        # Set up x-axis limits
        plt.xlim(0,self.x[len(self.x)-1])

        # Generate scatter plot
        plt.scatter(self.x,self.y) 

        # Set up axis labels
        plt.xlabel(self.x_label) 
        plt.ylabel(self.y_label) 

        # Generate plot of a regression model
        plt.plot(self.x, self.p(self.x), '-')

        # Show plot
        plt.show()
        
        # Save plot on png file
        plt.savefig(self.file_in[:len(self.file_in)-4]+"_"+self.aux_title+"_Pol_Degree_"+str(self.pol_deg)+".png")

def main():
    # Read space shuttle ascent data and modeling information
    file_in = input("\nType input file name => ")
    pol_deg = int(input("\nType degree of the polynomial => "))
    exp_var = int(input("\nType column of the explanatory variable => "))
    resp_var = int(input("\nType column of the response variable => "))
    
    # Instantiate an object of ModelAscent()
    m1 = ModelAscent(file_in,pol_deg,exp_var,resp_var)
    
    m1.read_data()
    m1.regression_model()
    m1.plot_data()
    m1.plot_model()

main()